/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************
#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes

uint8_t Can0MessageRAM[CAN0_MESSAGE_RAM_CONFIG_SIZE] __attribute__((aligned (32)));

// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main (void)
{
  uint8_t key_state = 1;
  uint8_t pre_key_state = 1;
  uint8_t loop_count = 0;

  uint8_t i;
  uint8_t flag_wait_to_send = false;

  uint32_t messageID = 0;
  uint8_t message[8];
  uint8_t messageLength = 0;
  uint32_t status = 0;

  uint8_t rx_message[8];
  uint32_t rx_messageID = 0;
  uint8_t rx_messageLength = 0;

  CAN_MSG_RX_FRAME_ATTRIBUTE msgFrameAttr = CAN_MSG_RX_DATA_FRAME;


  /* Initialize all modules */
  SYS_Initialize (NULL);
  OUTPUT_LED2_Set ();

  /* Set Message RAM Configuration */
  CAN0_MessageRAMConfigSet (Can0MessageRAM);

  while (true)
    {
      /* Maintain state machines of all polled MPLAB Harmony modules. */
      SYS_Tasks ();

      if (flag_wait_to_send == true)
        {
          CAN0_MessageTransmit (rx_messageID, rx_messageLength, message, CAN_MODE_NORMAL, CAN_MSG_ATTR_TX_FIFO_DATA_FRAME);
          flag_wait_to_send = false;
        }


      //## sending
      if (loop_count == 100)
        {
          if (INPUT_SW1_Get () == 0)
            key_state = 0;
          else
            key_state = 1;

          //## sending
          if ((key_state == 0) && (pre_key_state == 1))
            {
              OUTPUT_LED2_Toggle ();
              //== sending data
              if (CAN0_InterruptGet (CAN_INTERRUPT_TC_MASK))
                CAN0_InterruptClear (CAN_INTERRUPT_TC_MASK);
              for (i = 0; i < 8; i++)
                message[i] = 0x11 + i;
              messageLength = 8;
              messageID = 0x3aa;

              CAN0_MessageTransmit (messageID, messageLength, message, CAN_MODE_NORMAL, CAN_MSG_ATTR_TX_FIFO_DATA_FRAME);
            }
          pre_key_state = key_state;
          loop_count = 0;
        }

      //## receiving
      if (CAN0_InterruptGet (CAN_INTERRUPT_RF0N_MASK))
        {
          CAN0_InterruptClear (CAN_INTERRUPT_RF0N_MASK);

          /* Check CAN Status */
          status = CAN0_ErrorGet ();

          if (((status & CAN_PSR_LEC_Msk) == CAN_ERROR_NONE) || ((status & CAN_PSR_LEC_Msk) == CAN_ERROR_LEC_NC))
            {
              memset (rx_message, 0x00, sizeof (rx_message));

              /* Receive FIFO 0 New Message */
              if (CAN0_MessageReceive (&rx_messageID, &rx_messageLength, rx_message, 0, CAN_MSG_ATTR_RX_FIFO0, &msgFrameAttr) == true)
                {
                  flag_wait_to_send = true;
                  rx_messageID++;
                  for (i = 0; i < rx_messageLength; i++)
                    message[i] = rx_message[i] + 1;
                }
            }
        }
      loop_count++;
    }

  /* Execution should not come here during normal operation */

  return ( EXIT_FAILURE);
}


/*******************************************************************************
 End of File
*/

