/*******************************************************************************
  Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This file contains the "main" function for a project.

  Description:
    This file contains the "main" function for a project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state
    machines of all modules in the system
 *******************************************************************************/

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "definitions.h"                // SYS function prototypes

uint8_t Can0MessageRAM[CAN0_MESSAGE_RAM_CONFIG_SIZE] __attribute__((aligned (32)));

/* Application's state machine enum */
typedef enum
{
    APP_STATE_CAN_RECEIVE,
    APP_STATE_CAN_TRANSMIT,
    APP_STATE_CAN_IDLE,
} APP_STATES;

/* Variable to save Tx/Rx transfer status and context */
static CAN_ERROR status = 0;

/* Variable to save application state */
volatile static APP_STATES state = APP_STATE_CAN_IDLE;

static CAN_MSG_RX_FRAME_ATTRIBUTE msgFrameAttr = CAN_MSG_RX_DATA_FRAME;

bool STATUS_RX_PREPARE=true;
bool STATUS_RX_COMPLETED=false;
bool STATUS_TX_PREPARE=false;


void APP_CAN_TX_Callback(uintptr_t context)
{
  STATUS_RX_PREPARE=true;
  CAN0_InterruptClear (CAN_INTERRUPT_TC_MASK);
}


void APP_CAN_RX_Callback (uintptr_t context)
{
  /* Check CAN Status, Do something here */
  status = CAN0_ErrorGet ();
  
  STATUS_RX_COMPLETED=true;
  CAN0_InterruptClear (CAN_INTERRUPT_RF0N_MASK);
} 



// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main ( void)
{
  uint32_t messageID = 0;
  uint8_t  message[8];

  uint32_t rx_messageID=0;
  uint8_t  rx_message[8];
  uint8_t  rx_messageLength = 0;

  uint8_t  i=0;


  /* Initialize all modules */
  SYS_Initialize (NULL);

  /* Set Message RAM Configuration */
  CAN0_MessageRAMConfigSet (Can0MessageRAM);
  
  
  CAN0_RxCallbackRegister (APP_CAN_RX_Callback, (uintptr_t)APP_STATE_CAN_RECEIVE, CAN_MSG_ATTR_RX_FIFO0);
  CAN0_TxCallbackRegister (APP_CAN_TX_Callback, (uintptr_t) APP_STATE_CAN_TRANSMIT);
  messageID=0x123;

  while (true)
    {
      if(STATUS_RX_COMPLETED==true)
        {
          STATUS_RX_COMPLETED=false;
          
          message[0]=i;
          message[1]=i+1;
          i++;
          memcpy(&message[2], rx_message, rx_messageLength);
          STATUS_TX_PREPARE=true;
        }
      else if(STATUS_RX_PREPARE==true) // && STATUS_RX_COMPLETED==false
        {
          STATUS_RX_PREPARE=false;
          memset(rx_message, 0x00, 8);
          CAN0_MessageReceive (&rx_messageID, &rx_messageLength, rx_message, 0, CAN_MSG_ATTR_RX_FIFO0, &msgFrameAttr);
        }
      else if(STATUS_TX_PREPARE==true)
        {
          if(CAN0_MessageTransmit (messageID, rx_messageLength+2, message, CAN_MODE_NORMAL, CAN_MSG_ATTR_TX_FIFO_DATA_FRAME) == false)
             OUTPUT_LED2_Toggle();
          else
            STATUS_TX_PREPARE=false;                  
        }
        
    }
 
  /* Execution should not come here during normal operation */

  return ( EXIT_FAILURE);
}


/*******************************************************************************
 End of File
*/

